#!/usr/bin/env python3
# coding: utf-8

from __future__ import unicode_literals

from volkanic.system import CommandRegistry

entries = {
    'joker.superuser.pydir': 'pydir',
    'joker.superuser.pyentry': 'pyentry',
    'joker.superuser.unsource': 'unsource',
    'joker.superuser.cases': 'cases',
    'joker.superuser.dedup': 'dup',
    'joker.superuser.setop': 'setop',
}

registry = CommandRegistry(entries)

if __name__ == '__main__':
    registry()



#!/usr/bin/env python3
# coding: utf-8

from __future__ import print_function

import os
import subprocess
import sys


def install(*packages):
    subprocess.run([sys.executable, '-m', 'pip'] + list(packages))


def gettext_pipconf(repo='https://pypi.org/simple/'):
    if not repo.startswith('https:'):
        repo = "https://" + repo + '/simple/'
    lines = ["[global]", "index-url = {}".format(repo)]
    return os.linesep.join(lines)



#!/usr/bin/env python3
# coding: utf-8

import os
import re
import sys
from collections import defaultdict

from joker.cast.iterative import split
from joker.stream.shell import ShellStream
from joker.stream.utils import checksum

from joker.superuser import utils


# TODO: use a sha1sum txt file


def _thresh(size, minsize):
    if size >= minsize:
        return size


def get_sizefunc(minsize):
    sizefunc = utils.silent_function(os.path.getsize)
    if not minsize:
        return sizefunc
    return lambda p: _thresh(sizefunc(p), minsize)


@utils.silent_function
def chksum(path):
    return checksum(path).digest()


@utils.silent_function
def chksum_head(path, length=1024):
    return checksum(path, length=length).digest()


def _plural(grp):
    if len(grp) > 1:
        return grp


def _sort_plural(grp):
    if len(grp) > 1:
        grp.sort(reverse=True)
        return grp


def _subgroup(groups, kfunc, gfunc):
    subgrps = []
    for grp in groups:
        aux = defaultdict(list)
        for path in grp:
            k = kfunc(path)
            if k is not None:
                aux[k].append(path)
        batch = (gfunc(g) for g in aux.values())
        batch = (g for g in batch if g is not None)
        subgrps.extend(batch)
    return subgrps


def find_duplicates(dirs, minsize=0):
    all_files = []
    for path in dirs:
        all_files.extend(utils.find_regular_files(path))
    sizefunc = get_sizefunc(minsize)
    grps = _subgroup([all_files], sizefunc, _plural)
    grps = _subgroup(grps, chksum_head, _plural)
    grps = _subgroup(grps, chksum, _sort_plural)
    return grps


def parse_fdupes_output(path):
    with ShellStream.open(path).snl() as stream:
        for tup in split(stream, lambda x: not x):
            elements = tup[-1]
            if elements:
                yield elements


def check_existence(groups):
    for grp in groups:
        paths = [p for p in grp if os.path.isfile(p)]
        if len(paths) > 1:
            paths.sort(reverse=True)
            yield paths


def search_pattern(groups, pattern):
    for grp in groups:
        for path in grp:
            if re.search(pattern, path):
                yield grp
                break


def _make_pairs(paths, protect_regex):
    pairs = []
    for i, path in enumerate(paths):
        if protect_regex.match(path):
            pairs.append((0, path))
        else:
            pairs.append((1 + i, path))
    pairs.sort()
    return pairs


def prioritize(groups, protect=None):
    if protect:
        regex = re.compile(protect)
        for grp in groups:
            yield _make_pairs(grp, regex)
    else:
        for grp in groups:
            yield [(1 + i, p) for i, p in enumerate(grp)]


def _make_script(pairs):
    from shlex import quote
    pairs = list(pairs)
    if not any(p[0] for p in pairs):
        return
    path = pairs[0][1]
    print('#', quote(path))
    for prio, path in pairs[1:]:
        if prio:
            print('rm -f', quote(path))
        else:
            print('#', quote(path))
    print()


def _parse_args(prog, args):
    import argparse
    desc = 'find duplicating files recursively'
    pr = argparse.ArgumentParser(prog=prog, description=desc)

    pr.add_argument('-s', '--size', type=int, default=1,
                    help='minimum size for consideration')
    pr.add_argument('-p', '--pattern', metavar='regex',
                    help='select a subset of duplicating groups')
    pr.add_argument('-P', '--protect', metavar='regex',
                    help='prevent certain file from being deleted')

    pr.add_argument('-f', '--fdupes', help='an fdupes output file')
    pr.add_argument('dirs', nargs='*', help='target directories')
    return pr.parse_args(args)


def run(prog, args):
    ns = _parse_args(prog, args)
    if ns.fdupes:
        if ns.dirs:
            print('conflicting arguments: -f/--fdupes and dirs', file=sys.stderr)
        groups = parse_fdupes_output(ns.path)
        groups = check_existence(groups)
    else:
        groups = find_duplicates(ns.dirs, minsize=ns.size)
    if ns.pattern:
        groups = search_pattern(groups, ns.pattern)
    p_groups = prioritize(groups, ns.protect)
    for pgrp in p_groups:
        _make_script(pgrp)



#!/usr/bin/env python3
# coding: utf-8

import os
import re
import sys
from os.path import join

from joker.textmanip.tabular import format_help_section

from joker.superuser.utils import under_asset_dir

_nsinits = {
    'i': '__import__("pkg_resources").declare_namespace(__name__)',
    'p': '__path__ = __import__("pkgutil").extend_path(__path__, __name__)',
}


def _vk_join(d):
    return {k: join(v, k) for k, v in d.items()}


class ProjectDirectoryMaker(object):
    def __init__(self, nsp, pkg):
        """
        :param nsp: namespace
        :param pkg: package name
        """
        self.nsp = nsp.strip()
        self.pkg = pkg.strip()
        names = [s for s in [self.nsp, self.pkg]]
        self.names = ['-'.join(names)] + names
        testdir_name = '_'.join(['test'] + self.names[1:])
        self.common_dirs = {
            'docs': self.under_proj_dir('docs'),
            'templates': self.under_pkg_dir('templates'),
            'test': self.under_proj_dir(testdir_name),
        }
        self.common_files = _vk_join({
            '.gitignore': self.names[0],
            'requirements.txt': self.names[0],
            'MANIFEST.in': self.names[0],
            'setup.py': self.names[0],
            '__init__.py': self.under_pkg_dir(),
        })
        if self.nsp:
            p = join(self.names[0], self.nsp, '__init__.py')
            self.common_files['nsinit'] = p

    @classmethod
    def parse(cls, name):
        """
        :param name: e.g. "volkanic", "joker.superuser"
        :return: a ProjectDirectoryMaker instance
        """
        mat = re.match(r'([_A-Za-z]\w+\.)?([_A-Za-z]\w+)', name)
        if not mat:
            raise ValueError('invalid pakcage name: ' + repr(name))
        return cls(mat.group(1)[:-1] or '', mat.group(2))

    def under_proj_dir(self, *paths):
        return join(self.names[0], *paths)

    def under_pkg_dir(self, *paths):
        return join(*(self.names + list(paths)))

    def locate(self, name):
        if name.endswith('/'):
            return self.common_dirs.get(name[:-1])
        return self.common_files.get(name)

    def make_dirs(self):
        for path in self.common_dirs.values():
            os.makedirs(path, exist_ok=True)
        for path in self.common_files.values():
            open(path, 'a').close()

    def sub(self, text):
        text = text.replace('__sus_h_name', self.names[0])
        text = text.replace('__sus_i_name', '.'.join(self.names[1:]))
        text = text.replace('__sus_u_name', '_'.join(self.names[1:]))
        text = text.replace('__sus_namespace', self.nsp)
        text = text.replace('__sus_package', self.pkg)
        return text

    def gettext_setup(self, template_path):
        template_path = template_path or under_asset_dir('setup.txt')
        code = open(template_path).read()
        return self.sub(code)

    def gettext_manifest(self):
        templates_path = os.path.sep.join(self.names[1:] + ['templates'])
        return os.linesep.join([
            'include requirements.txt',
            'exclude {}/*'.format(self.common_dirs['test']),
            'recursive-include {} *.html'.format(templates_path)
        ]) + os.linesep

    def write(self, name, content):
        path = self.common_files.get(name)
        if path and content:
            with open(path, 'a') as fout:
                fout.write(content + os.linesep)

    def write_nsinit(self, approach):
        self.write('nsinit', _nsinits.get(approach))

    def write_version_variable(self):
        self.write('__init__.py', "__version__ = '0.0'")

    def write_requirements(self, lines):
        self.write('requirements.txt', os.linesep.join(lines))

    def write_setup(self, template_path=''):
        self.write('setup.py', self.gettext_setup(template_path))

    def write_manifest(self):
        self.write('MANIFEST.in', self.gettext_manifest())

    def write_gitignore(self, path=''):
        path = path or under_asset_dir('gitignore.txt')
        self.write('.gitignore', open(path).read())


def make_project(name, setup, gitignore, require, ns_approach):
    name, query = (name.split('%', maxsplit=1) + [''])[:2]
    if query == 'nsinit':
        print(_nsinits.get(ns_approach, ''))
        return
    if query == 'gitignore':
        print(open(under_asset_dir('gitignore.txt')).read())
        return

    mkr = ProjectDirectoryMaker.parse(name)
    if query == 'sub':
        print(mkr.sub(sys.stdin.read()))
        return
    if query == 'setup':
        print(mkr.gettext_setup(setup))
        return
    if query == 'manifest':
        print(mkr.gettext_manifest())
        return

    try:
        return print(mkr.locate(query) + '')
    except TypeError:
        pass

    if query:
        raise ValueError('invalid query: ' + repr(query))

    mkr.make_dirs()
    mkr.write_setup(setup)
    mkr.write_gitignore(gitignore)
    mkr.write_requirements(require or [])
    mkr.write_manifest()
    mkr.write_nsinit(ns_approach)
    mkr.write_version_variable()


def run(prog=None, args=None):
    import argparse
    desc = 'Generate a python project structure'
    # s = ' (case sensitive)'
    epilog = '\n'.join([
        format_help_section('Namespace package approaches', _nsinits),
        'About namespace packages:',
        '  https://packaging.python.org/guides/packaging-namespace-packages/',
    ])
    pr = argparse.ArgumentParser(
        prog=prog, description=desc,  epilog=epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter)

    pr.add_argument('-n', '--ns-approach', choices=['i', 'p', 'e'],
                    default='i', help='namespace package approach')

    pr.add_argument('-s', '--setup', metavar='template_setup.py',
                    help='path to a custom setup.py template')

    pr.add_argument('-g', '--gitignore', metavar='template_gitignore.txt',
                    help='path to a custom .gitignore template')

    pr.add_argument('-r', '--require', action='append', metavar='PACKAGE',
                    help='example: -r six -r numpy>=1.0.0')

    pr.add_argument('name', metavar='[NAMESPACE.]PACKAGE_NAME[%QUERY]')
    ns = pr.parse_args(args)

    try:
        make_project(**vars(ns))
    except ValueError as e:
        print(str(e), file=sys.stderr)



#!/usr/bin/env python3
# coding: utf-8

import os
import re

from setuptools import setup, find_packages

# CAUTION:
# Do NOT import your package from your setup.py

namespace = 'joker'
package_name = 'superuser'
description = ''


def read(filename):
    with open(filename) as f:
        return f.read()


def version_find():
    if namespace:
        path = '{}/{}/__init__.py'.format(namespace, package_name)
    else:
        path = '{}/__init__.py'.format(package_name)
    root = os.path.dirname(__file__)
    path = os.path.join(root, path)
    regex = re.compile(
        r'''^__version__\s*=\s*('|"|'{3}|"{3})([.\w]+)\1\s*(#|$)''')
    with open(path) as fin:
        for line in fin:
            line = line.strip()
            if not line or line.startswith('#'):
                continue
            mat = regex.match(line)
            if mat:
                return mat.groups()[1]
    raise ValueError('__version__ definition not found')


config = {
    'name': package_name,
    'version': version_find(),
    'description': '' + description,
    'keywords': 'sysadmin',
    'url': 'https://github.com/frozflame',
    'author': 'frozflame',
    'author_email': 'anonym@example.com',
    'license': "GNU General Public License (GPL)",
    'packages': find_packages(exclude=['test_*']),
    'zip_safe': False,
    'install_requires': read("requirements.txt"),
    'entry_points': {'console_scripts': ['sus = joker.superuser.main:registry'], },
    'classifiers': [
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    # ensure copy static file to runtime directory
    'include_package_data': True,
}

if namespace:
    config['name'] = '{}-{}'.format(namespace, package_name)
    config['namespace_packages'] = [namespace]


setup(**config)

